import numpy as np

#bring in data from files
data_train = np.loadtxt(open("train.txt"))
data_test = np.loadtxt(open("test.txt"))
print("Length of training set: %d " % len(data_train))
print("Length of test set: %d " % len(data_test))
#can't define 3rd column as integer, or it will collapse to 1D array and next line won't work?
Xtrn = data_train[:, 0:2]  # first 2 columns of training set
Ytrn = data_train[:, 2]  # last column, 1/0 labels
Xtst = data_test[:, 0:2]  # first 2 columns of test set
Ytst = data_test[:, 2]  # last column, 1/0 labels

#get the first 2 columns of the training set where the 3rd column is 0
Xtrn_0 = data_train[data_train[:,2] == 0, 0:2]
#get the first 2 columns of the training set where the 3rd column is 1
Xtrn_1 = data_train[data_train[:,2] == 1, 0:2]

print("\nNumber of Class 0 items in training set: %d " % len(Xtrn_0))
print("Number of Class 1 items in training set: %d " % len(Xtrn_1))

#get the means for each column in class 0
col_mean0 = [0,0]
col_mean0[0] = sum(Xtrn_0[:,0])/len(Xtrn_0)
col_mean0[1] = sum(Xtrn_0[:,1])/len(Xtrn_0)

#build covariance matrix for class 0 (hard-coding as 2x2 for now)
E0 = [[0 for a in range(2)] for b in range(2)]
for i in range(0,2):
   for j in range(0,2):
       for k in range (0, len(Xtrn_0)):
           #sum up the (x-mux)(y-muy) values
            E0[i][j] += (Xtrn_0[k][i] - col_mean0[i])*(Xtrn_0[k][j] - col_mean0[j])
#divide each item in E by N-1
E0[0][0] = E0[0][0] / (len(Xtrn_0)-1) 
E0[0][1] = E0[0][1] / (len(Xtrn_0)-1)
E0[1][0] = E0[1][0] / (len(Xtrn_0)-1)
E0[1][1] = E0[1][1] / (len(Xtrn_0)-1)
'''
print("Covariance Matrix Class 0")
print(E0)
print("Mean of Class 0 Column 0: %f" % col_mean0[0])
print("Mean of Class 0 Column 1: %f" % col_mean0[1])
'''

#get the means for each column in class 1
col_mean1 = [0,0]
col_mean1[0] = sum(Xtrn_1[:,0])/len(Xtrn_1)
col_mean1[1] = sum(Xtrn_1[:,1])/len(Xtrn_1)

#build covariance matrix for class 0 (hard-coding as 2x2 for now)
E1 = [[0 for a in range(2)] for b in range(2)]
for i in range(0,2):
   for j in range(0,2):
       for k in range (0, len(Xtrn_1)):
           #sum up the (x-mux)(y-muy) values
            E1[i][j] += (Xtrn_1[k][i] - col_mean1[i])*(Xtrn_1[k][j] - col_mean1[j])
#divide each item in E by N-1
E1[0][0] = E1[0][0] / (len(Xtrn_1)-1) 
E1[0][1] = E1[0][1] / (len(Xtrn_1)-1)
E1[1][0] = E1[1][0] / (len(Xtrn_1)-1)
E1[1][1] = E1[1][1] / (len(Xtrn_1)-1)
'''
print("Covariance Matrix Class 1")
print(E1)
print("Mean of Class 1 Column 0: %f" % col_mean1[0])
print("Mean of Class 1 Column 1: %f" % col_mean1[1])
'''

#d-dimensional gaussian for class 0
Gauss0_0 = []
Gauss0_1 = []
Xprob = []
#get the probability that each X-pair (from the TRAINING file) lies within a particular class
import random
for x in range(len(Xtrn)):
    #max(p(xm|ci)p(ci)) #NOTE: p(ci) is the same for both classes in this case, 0.5
   Gauss0_0.append(((1/((2*np.pi) ** (2/2)))*(1/(np.power(np.linalg.det(E0),1/2)))*np.exp((-1/2)*np.matrix([i - j for i, j in zip(Xtrn[x], col_mean0)])*np.linalg.inv(E0)*np.transpose(np.matrix([i - j for i, j in zip(Xtrn[x], col_mean0)]))))*0.5)
   Gauss0_1.append(((1/((2*np.pi) ** (2/2)))*(1/(np.power(np.linalg.det(E1),1/2)))*np.exp((-1/2)*np.matrix([i - j for i, j in zip(Xtrn[x], col_mean1)])*np.linalg.inv(E1)*np.transpose(np.matrix([i - j for i, j in zip(Xtrn[x], col_mean1)]))))*0.5)
   if Gauss0_0[x] > Gauss0_1[x]:
       Xprob.append(0)
   elif Gauss0_0[x] < Gauss0_1[x]:
       Xprob.append(1) 
   else:
       Xprob[i][2] = random.randint(0, 1) #if both probabilities are equal, randomly assign
#check probable class against the true y
count_correct = 0
for x in range(len(Xprob)):
    if Xprob[x] == Ytrn[x]:
        count_correct +=1

print("\nTraining Set Correctly classified: %d" % count_correct)
print("Training Set Incorrectly classified: %d" % (len(Xprob)-count_correct))

#get the probability that each X-pair (from the TEST file) lies within a particular class
Gauss0_0 = []
Gauss0_1 = []
Xprob = []
for x in range(len(Xtst)):
    #max(p(xm|ci)p(ci)) #NOTE: p(ci) is the same for both classes in this case, 0.5
   Gauss0_0.append(((1/((2*np.pi) ** (2/2)))*(1/(np.power(np.linalg.det(E0),1/2)))*np.exp((-1/2)*np.matrix([i - j for i, j in zip(Xtst[x], col_mean0)])*np.linalg.inv(E0)*np.transpose(np.matrix([i - j for i, j in zip(Xtst[x], col_mean0)]))))*0.5)
   Gauss0_1.append(((1/((2*np.pi) ** (2/2)))*(1/(np.power(np.linalg.det(E1),1/2)))*np.exp((-1/2)*np.matrix([i - j for i, j in zip(Xtst[x], col_mean1)])*np.linalg.inv(E1)*np.transpose(np.matrix([i - j for i, j in zip(Xtst[x], col_mean1)]))))*0.5)
   if Gauss0_0[x] > Gauss0_1[x]:
       Xprob.append(0)
   elif Gauss0_0[x] < Gauss0_1[x]:
       Xprob.append(1) 
   else:
       Xprob[i][2] = random.randint(0, 1) #if both probabilities are equal, randomly assign
#check probable class against the true y
count_correct = 0
for x in range(len(Xprob)):
    if Xprob[x] == Ytst[x]:
        count_correct +=1

print("\nTest Set Correctly classified: %d" % count_correct)
print("Test Set Incorrectly classified: %d" % (len(Xprob)-count_correct))
